package main

import (
	"fmt"
	"github.com/gotidy/copy"
	"github.com/urfave/cli/v2"
	"os"
)

const VERSION = "v0.1.0"

var App *cli.App

func main() {
	sflags := []cli.Flag{
		&cli.StringFlag{
			Name:    "out",
			Aliases: []string{"o"},
			Usage:   "Specify output wav file. Default value: 'tape.wav'",
			Value:   "tape.wav",
		},
		&cli.StringFlag{
			Name:    "machine",
			Aliases: []string{"mt"},
			Usage:   HELP_MACHINE_TYPE,
			Value:   "mz-700",
		},
		&cli.BoolFlag{
			Name:    "invert-polarity",
			Aliases: []string{"inv"},
			Usage:   "Invert audio polarity. Default value: false",
			Value:   false,
		},
		&cli.StringFlag{
			Name:     "file-attrib",
			Aliases:  []string{"fa"},
			Usage:    HELP_FILE_ATTRIBUTE,
			Required: true,
		},
		&cli.StringFlag{
			Name:     "file-name",
			Aliases:  []string{"fn"},
			Usage:    "Filename for tape file. (Not the input file name)",
			Required: true,
		},
		&cli.StringFlag{
			Name:     "load-addr",
			Aliases:  []string{"la"},
			Usage:    "Address for load.",
			Required: true,
		},
		&cli.StringFlag{
			Name:     "exec-addr",
			Aliases:  []string{"ea"},
			Usage:    "Address that will be jump to when tape loaded. If not specified, will not execute.",
			Required: false,
		},
		&cli.StringFlag{
			Name:     "comment",
			Aliases:  []string{"cm"},
			Usage:    "Tape file comment. If not specified, will be blank.",
			Required: false,
		},
		&cli.StringFlag{
			Name:     "in",
			Aliases:  []string{"i"},
			Usage:    "Input binary file.",
			Required: true,
		},
		&cli.BoolFlag{
			Name:    "verbose",
			Aliases: []string{"v"},
			Usage:   "Print out verbose information.",
			Value:   false,
		},
	}
	var eyflags []cli.Flag
	copier := copy.New()
	copier.Copy(&eyflags, &sflags)
	eyfout := &cli.StringFlag{
		Name:     "yaml",
		Aliases:  []string{"yml"},
		Usage:    "Output yaml file",
		Required: true,
	}
	eyflags = append(eyflags, eyfout)
	App = &cli.App{
		Name:        "bin2mz700wav",
		Usage:       "Convert binary file into SHARP MZ-700 Tape wave file.",
		Version:     VERSION,
		Description: "A simple convert program for converting binary file into SHARP MZ-700 Tape wave file.",
		Commands: []*cli.Command{
			{
				Name:    "simple",
				Aliases: []string{"s"},
				Usage:   "Convert with specified config file",
				Flags:   sflags,
				Action:  CmdSimple,
			},
			{
				Name:    "from-yaml",
				Aliases: []string{"fy"},
				Usage:   "Convert with specified yaml config file",
				Flags: []cli.Flag{
					&cli.StringFlag{
						Name:     "in",
						Aliases:  []string{"i", "y", "yml", "yaml"},
						Usage:    "Input yaml file.",
						Required: true,
					},
				},
				Action: CmdYamlConfig,
			},
			{
				Name:    "example-yaml",
				Aliases: []string{"ey"},
				Usage:   "Generate example yaml config file",
				Flags:   eyflags,
				Action:  CmdExampleYaml,
			},
			{
				Name:    "convert-ascii",
				Aliases: []string{"ca"},
				Usage:   "convert text file between MZ-700 ASCII and standard ASCII",
				Subcommands: []*cli.Command{
					{
						Name:    "mz2ascii",
						Aliases: []string{"m2a"},
						Usage:   "",
						Action:  CmdConvertMZ2ASCII,
						Flags: []cli.Flag{
							&cli.StringFlag{
								Name:     "in",
								Aliases:  []string{"i"},
								Usage:    "Input MZ text file.",
								Required: true,
							},
							&cli.StringFlag{
								Name:     "out",
								Aliases:  []string{"o"},
								Usage:    "Output ASCII text file. Override if file exists, create if file not exist.",
								Required: true,
							},
							&cli.BoolFlag{
								Name:    "ignore-non-disp-char",
								Aliases: []string{"ign"},
								Usage:   "Ignore non-display ASCII characters. (See documentation)",
								Value:   false,
							},
							&cli.BoolFlag{
								Name:    "use-cr-lf",
								Aliases: []string{"crlf"},
								Usage:   "Convert 0x0D into \\r\\n. (See documentation)",
								Value:   false,
							},
						},
					},
					{
						Name:    "ascii2mz",
						Aliases: []string{"a2m"},
						Usage:   "",
						Action:  CmdConvertASCII2MZ,
						Flags: []cli.Flag{
							&cli.StringFlag{
								Name:     "in",
								Aliases:  []string{"i"},
								Usage:    "Input ASCII text file.",
								Required: true,
							},
							&cli.StringFlag{
								Name:     "out",
								Aliases:  []string{"o"},
								Usage:    "Output MZ text file. Override if file exists, create if file not exist.",
								Required: true,
							},
							&cli.BoolFlag{
								Name:    "ignore-non-disp-char",
								Aliases: []string{"ign"},
								Usage:   "Ignore non-display ASCII characters. (See documentation)",
								Value:   false,
							},
							&cli.BoolFlag{
								Name:    "ignore-return",
								Aliases: []string{"ir"},
								Usage:   "Ignore \\r (0x0D). (See documentation)",
								Value:   false,
							},
						},
					},
				},
			},
		},
		Flags:                nil,
		EnableBashCompletion: true,
		HideHelp:             false,
		HideHelpCommand:      false,
		HideVersion:          false,
		Authors: []*cli.Author{
			{
				Name:  "zry",
				Email: "admin@z-touhou.org",
			},
		},
	}
	err := App.Run(os.Args)
	if err != nil {
		fmt.Println("failed run command: ", err.Error())
	}
}
