package main

import (
	"fmt"
	mz700_tape_wav_gen "git.swzry.com/zry/mz700-tape-wav-gen"
	"io/ioutil"
	"os"
)

func ProcessingConvert(cfg *ConfigDef, verbose bool) error {
	fname := cfg.OutputWav
	fmt.Println("Output WAV File: ", fname)
	fout, err := os.OpenFile(fname, os.O_WRONLY|os.O_TRUNC|os.O_CREATE, 0666)
	if err != nil {
		return fmt.Errorf("failed open output file '%s': %s", fname, err)
	}
	if verbose {
		fmt.Println("Output WAV file opened.")
		fmt.Println("Initializing SHARP MZ Tape WAV Generator object...")
	}
	var machine mz700_tape_wav_gen.MachineType
	switch cfg.MachineType {
	case "mz-700":
		machine = mz700_tape_wav_gen.MACHINE_MZ_700
		break
	case "mz-80k":
		machine = mz700_tape_wav_gen.MACHINE_MZ_80K
		break
	case "mz-80a":
		machine = mz700_tape_wav_gen.MACHINE_MZ_80A
		break
	case "mz-80b":
		machine = mz700_tape_wav_gen.MACHINE_MZ_80B
		break
	case "mz-800":
		machine = mz700_tape_wav_gen.MACHINE_MZ_800
		break
	default:
		return fmt.Errorf("invalid machine type '%s'", cfg.MachineType)
	}
	fmt.Println("Target Machine Type: ", cfg.MachineType)
	if cfg.InvertPolarity {
		fmt.Println("Polarity: Inverted")
	} else {
		fmt.Println("Polarity: Normal")
	}
	stwg, err := mz700_tape_wav_gen.NewStandardTapeWaveGenerator(fout, machine, cfg.InvertPolarity)
	defer func() {
		err2 := stwg.Close()
		if err2 != nil {
			fmt.Printf("failed close file '%s': %s", fname, err.Error())
		}
	}()
	if err != nil {
		return fmt.Errorf("failed create object StandardTapeWaveGenerator: %s", err.Error())
	}
	if verbose {
		fmt.Println("Generator object initialization OK.")
	}
	tfcount := len(cfg.TapeFiles)
	if tfcount == 0 {
		fmt.Println("No tape file will be put into tape WAV.")
		err = fout.Close()
		if err != nil {
			return fmt.Errorf("failed to close opened WAV file: %s", err.Error())
		}
		fmt.Println("End.")
		return nil
	}
	fmt.Printf("%d Tape file(s) will be put into tape WAV.\n", tfcount)
	for i, v := range cfg.TapeFiles {
		if verbose {
			fmt.Printf("Processing File %d ...\n", i+1)
		}
		fa, sfa, err := ParseFileAttrib(v.FileAttribute)
		if err != nil {
			return fmt.Errorf("error @ Tape File %d ('%s'): %s", i+1, v.FileName, err.Error())
		}
		if verbose {
			fmt.Printf("\tFile Attribute: 0x%02X (%s) \n", fa, sfa)
			fmt.Println("\tFile Name: ", v.FileName)
			fmt.Printf("\tLoad Address: 0x%04X\n", v.LoadAddress)
			if v.ExecuteAfterLoad {
				fmt.Printf("\tExecute Option:  Will Jump To 0x%04X After Load.\n", v.ExecuteAddress)
			} else {
				fmt.Println("\tExecute Option: Back to Monitor.")
			}
			if v.UseRawCommentFromFile {
				fmt.Printf("\tComment Option: Raw from File '%s'\n", v.RawCommentFileName)
			} else {
				fmt.Println("\tComment Option: Text")
				fmt.Println("\t==== Comment ====")
				fmt.Println(AddTabBeforeEachLine(v.StringComment))
				fmt.Println("\t========")
			}
		}
		var rawcmdat []byte
		if v.UseRawCommentFromFile {
			rawcmdat, err = ioutil.ReadFile(v.RawCommentFileName)
			if err != nil {
				return fmt.Errorf("error @ Tape File %d ('%s'): raw comment file read: %s", i+1, v.FileName, err.Error())
			}
		}
		var ftdata []byte
		switch v.DateSource {
		case "bin-file":
			ftdata, err = DS_BinFile(v.FileDataSource)
			break
		case "ascii-text-file":
			ftdata, err = DS_ASCIITextFile(v.ASCIITextFileDataSource, v.IgnoreNonDisplayChar, v.IgnoreCR)
			break
		case "raw":
			ftdata, err = DS_Raw(v.RawData)
			break
		default:
			return fmt.Errorf("unsupport data source '%s'", v.DateSource)
		}
		if verbose {
			fmt.Println("\tData Source Type: ", v.DateSource)
			fmt.Println("\tData Length: ", len(ftdata))
		}
		var scm string
		var bcm []byte
		if v.UseRawCommentFromFile {
			bcm = rawcmdat
			scm = ""
		} else {
			scm = v.StringComment
			bcm = nil
		}
		tf := &mz700_tape_wav_gen.StandardTapeFile{
			FileAttribute:    fa,
			Filename:         v.FileName,
			LoadAddress:      v.LoadAddress,
			ExecuteAfterLoad: v.ExecuteAfterLoad,
			ExecuteAddress:   v.ExecuteAddress,
			IsStringComment:  !v.UseRawCommentFromFile,
			StringComment:    scm,
			BinaryComment:    bcm,
			File:             ftdata,
		}
		err = stwg.WriteStandardFile(tf)
		if err != nil {
			return fmt.Errorf("error @ Tape File %d ('%s'): %s", i+1, v.FileName, err.Error())
		}
		stwg.WriteSilent1Second()
		stwg.WriteSilent1Second()
		stwg.WriteSilent1Second()
	}
	fmt.Println("Done.")
	return nil
}

func DS_BinFile(filename string) ([]byte, error) {
	fdat, err := ioutil.ReadFile(filename)
	if err != nil {
		return nil, err
	} else {
		return fdat, nil
	}
}

func DS_ASCIITextFile(filename string, ign bool, icr bool) ([]byte, error) {
	fdat, err := ioutil.ReadFile(filename)
	if err != nil {
		return nil, err
	}
	rdat := _ASCII2MZ700(fdat, ign, icr)
	return rdat, nil
}

func DS_Raw(in *[]byte) ([]byte, error) {
	if in == nil {
		return []byte{}, nil
	}
	return *in, nil
}
